### USER-DEFINED SECTION; edit these values ###
# Specify case and control sample sizes below
n1 = 2000
n2 = 2000
# Specify a table of effect distribution below
# plausible odds ratios (Odds)
# and the number of loci with that effect (Nloc)
Odds = c(1.15, 1.5, 1.75, 1.8, 2)
Nloc = c(50,   20,  10,    3,   1)
### END OF USER-DEFINED SECTION ###

# need the expectation of lodds*P(1-P); P is MAF
# for each bin, will assume p ~ Uniform(0,1)
# P*(1-P) ~ 2/sqrt(1-4*p); 0 < p < 1/4
# Then E[P*(1-P)] = 1/6
eff.ta = (1/6) * log(Odds)^2
numlo.ta = Nloc
fre.ta <- numlo.ta/sum(numlo.ta)
n.ta <- 1/mean(1/c(n1,n2))
ncp.ta <- n.ta * eff.ta

sum.ta <- sum(numlo.ta)
nn.ta <- round(sum.ta)

mh <- sum(ncp.ta * numlo.ta/sum.ta)
vh <- (sum(ncp.ta^2 * numlo.ta) -
       sum(ncp.ta * numlo.ta)^2/sum.ta)/(sum.ta-1)

# Method of moments estimates
mShape <- mh^2/vh
mScale <- vh/mh
At <- max(ncp.ta)+10

# random numbers from right-truncated gamma density
frnd.dist <- function(n, shape,scale,trun) {
    x <- runif(n)
    xx <- x*pgamma(trun, shape=shape, scale=scale)
    qgamma(xx, shape=shape, scale=scale)
}

Qua <- rep(ncp.ta, numlo.ta)
### force shape and fit scale by quantiless
qShape <- 1
qScale <- Start <- 1
maxScale <- 200
for(j in seq(Start, maxScale, by=0.5)) {
  qDif <- 0
  for(i in seq(0.1, 0.9, by=0.1)) {
    q1 <- as.numeric(quantile(Qua, i))
    q2 <- qgamma(i, shape=qShape, scale=j)
    # uncomment next 2 lines to fit to a truncated gamma (slow) 
    # trun.sample <- frnd.dist(1e4, shape=qShape,scale=j, trun=At)
    # q2 <- as.numeric(quantile(trun.sample, i))
    qDif <- (q1-q2)^2 + qDif
  }
  if(j <= Start) {
    minDif <- qDif
  } else {
    if(qDif < minDif) { minDif <- qDif; qScale <- j }
  }
}

cat("Method of moments estimates: Shape =", mShape, "Scale =", mScale, "\n")
cat("Quantile fit: forcing Shape =", qShape, "Scale estimate =", qScale, "\n")
if(qScale >= maxScale) {
  cat("Warning: maxScale reached, edit the script to increase maxScale value\n")
}
