# Time-stamp: <2011-11-17 22:20:37 zaykind> (written by Dmitri Zaykin)

###### USER-DEFINED SECTION
# Set Relative.Risk to zero or a negative value if want to use power instead
Relative.Risk <- 1.5
Power <- 0.94 # (at alpha = 5%)
Risk.Allele.Frequency <- 0.15
# set Base.Allele.Risk to zero if willing
# to assume rare disease
Base.Allele.Risk <- 0
Num.Cases <- 5000
Num.Controls <- 2000
Num.SNPs <- 5e5
Wanted.Prob.To.Capture <- 0.5
# if Num.TruePositives > 1, output probability
# of capturing at least one of them
Num.TruePositives <- 1
Extremal.Index <- 1
###### END OF USER-DEFINED SECTION

GntFrq <- function(f, rsk.A, rsk.a) {
  pdAA <- rsk.A^2
  pdAa <- rsk.A * rsk.a
  pdaa <- rsk.a^2
  h <- 1-f
  prev <- f*f*pdAA + 2*f*h*pdAa + h*h*pdaa
  cat("Population prevalence attributable to this SNP =", prev, "\n")
  pAAd <- f*f*pdAA / prev
  pAad <- 2*f*h*pdAa / prev
  paad <- h*h*pdaa / prev
  nprev <- 1-prev
  pAAc <- f*f*(1-pdAA) / (1-prev)
  pAac <- 2*f*h*(1-pdAa) / (1-prev)
  paac <- h*h*(1-pdaa) / (1-prev)
  c(pAAd, pAad, paad, pAAc, pAac, paac)
}

GntFrq.Rare <- function(f, Relative.Risk) {
  h <- 1-f
  den <- Relative.Risk^2 * f*f + Relative.Risk * 2*f*h + h*h
  pAAd <- Relative.Risk^2 * f*f / den
  pAad <- 2*f*h * Relative.Risk / den
  paad <- h*h / den

  pAAc <- f*f
  pAac <- 2*f*h
  paac <- h*h

  c(pAAd, pAad, paad, pAAc, pAac, paac)
}

Trend.Chi <- function(x, y) {
  nd <- (x[1]+x[2]+x[3])
  nh <- (y[1]+y[2]+y[3])
  n <- nd + nh
  n0 <- x[1]+y[1]
  n1 <- x[2]+y[2]
  n2 <- x[3]+y[3]
  nom <- n*(n*(x[2]+2*x[3]) - nd*(n1+2*n2))^2
  den <- nd*nh*(n*(n1+4*n2) - (n1+2*n2)^2)
  xx <- nom/den
}

Frq <- function(x) { (2*x[1]+x[2]) / (2*(x[1]+x[2]+x[3])) }

NonCentr <- function(xy, rr, ss) {
  n <- rr+ss
  pA <- xy[1:3]
  pN <- xy[4:6]
  x <- c(0,1,2)
  num <- (sum(x*(pA - pN)))^2
  den1 <- sum(x^2*(rr*pA + ss*pN))
  den2 <- (sum(x*(rr*pA + ss*pN)))^2 / n
  rr*ss*(num/(den1-den2))
}

cdf <- function(a, nc, df) {
  1 - pchisq(qchisq(1-a, df, ncp=0), df, ncp=nc)
}

cdfRoot <- function(nc, df, a, b) { 
    cdf(a,nc,df) - b 
}

getNonCentr <- function(df, a, b) {
   nc <- uniroot(cdfRoot, c(0, 2000), df=df, a=a, b=b)$root
   return (nc)
}

Fy <- function(y, df, nc) {
   1-pchisq(qchisq(1-y,df=df),df=df,ncp=nc)
}

min.Fy <- function(y, df, nc, m) {
   1-(1-Fy(y,df,nc))^m
}

eff.rank <- function(i, t, n) {
  (i - 1)*(t*n - 1)/(n - 1) + 1
}

Pr.minY <- function(i,L,th,df,nc,m) {
  ie <- eff.rank(i, th, L)
  min.Fy(qbeta(0.5, ie, th*L-ie+1),  df, nc, m)
}

df <- 1
if(Relative.Risk > 0) {
  rsk.A <- Relative.Risk * Base.Allele.Risk
  if(rsk.A > 1) stop(" Relative.Risk * Base.Allele.Risk > 1")

  if(Base.Allele.Risk > 0) {
    xy <- GntFrq(Risk.Allele.Frequency, rsk.A, Base.Allele.Risk)
  } else {
    xy <- GntFrq.Rare(Risk.Allele.Frequency, Relative.Risk)
  }
  nc <- NonCentr(xy, Num.Cases, Num.Controls)
} else {
  alpha <- 0.05
  nc <- getNonCentr(df, alpha, Power)
}

cat("Probability to capture an association\n")
for(i in 1:Num.SNPs) {
  rnk.Prob <- Pr.minY(i, Num.SNPs, Extremal.Index, df, nc, Num.TruePositives)
  if(!(i%%20)) cat("... among first", i, "best results =", rnk.Prob, "\n")
  if(rnk.Prob >= Wanted.Prob.To.Capture) break;
}
cat("... among first", i, "best results =", rnk.Prob, "\n")
cat("number of SNPs to follow up on for", 100*Wanted.Prob.To.Capture,
    "% probability =", i, "\n")

Frq <- function(x) { (2*x[1]+x[2]) / (2*(x[1]+x[2]+x[3])) }
pp <- Frq(xy[1:3])
qq <- Frq(xy[4:6])
poolfr <- (pp*(2*Num.Cases) + qq*(2*Num.Controls)) / (2*Num.Cases + 2*Num.Controls)
allelic1.Ncp <- (pp - qq)^2 / ((1 - poolfr)*poolfr * (1/(2*Num.Cases) + 1/(2*Num.Controls)))
allelic2.Ncp <- (pp - qq)^2 / ((1 - pp)*pp/(2*Num.Cases) + (1 - qq)*qq/(2*Num.Controls))
