### user-defined section; edit these values ###
source("Get-noncentralities-from-table.r") # make use of tabulated effect size distribution
# comment out previous line and
# uncomment next line if want to fit effect size distribution parameters from GWAS p-values
# source("Estimate-effects-distribution.r")
DataFile = "pvalues.csv"
k = 1e6 # total number of tests, "L+m"
# make up the output file name that starts with "OUT_", contains
# current time, and ends with the string that is the same as the input
Output.File = paste("OUT_", Sys.Date(), "_", DataFile, sep="")
### end of user-defined section ###

d <- read.csv(DataFile, as.is=TRUE)
d = d[order(d$Pvalue),]
Params <- c(qShape, qScale, At) # (shape, scale, truncation)
m <- sum(Nloc)

# right-truncated gamma density
fd.dist <- function(x, Params) {
  dgamma(x, shape=Params[1], scale=Params[2]) /
    pgamma(Params[3], shape=Params[1], scale=Params[2])
}

fy.int <- function(nc, ar, Params) {
  fd.dist(nc, Params) * dchisq(ar, df=1, ncp=nc)
}

# p-value density for true effects
fy.marg <- function(pval, Params) {
  ar <- qchisq(pval, df=1, lower.tail=FALSE)
  marg <- integrate(fy.int, 0, Params[3], ar,
     Params)$value / dchisq(ar, df=1)
  return (marg)
}

# posterior Pr(H_0 | p-value)
post0.marg <- function(pval, p0, Prior) {
  marg <- fy.marg(pval, Prior)
  (1 / (1 + (1-p0)/p0 * marg))
}

p = d$Pvalue
u = length(d$Pvalue)
po <- cu <- array(NA, u)
emp <- 0
for(j in 1:u) {
  po[j] = post0.marg(p[j], 1-m/k, Params)
  emp = emp + po[j]
  cu[j] = emp/j
}
emp <- emp/u
cat("Estimated proportion of true associations =", 1-emp, "\n")

d$Prob.False = format(po,digits=3)
d$rFDR = format(cu,digits=3)

write.csv(d, file = Output.File, row.names = FALSE)
